@@ -59,11 +59,13 @@ def main():
                                 the default""")
     parser.add_argument('-b', '--build',
                         default=None, action='store',
-                        help=
-                            'Override the current build number just this once')
+                        help="""Override the current build number just
+                                this once""")
     parser.add_argument('-c', '--channel',
                         default=None, action='store',
-                        help='Override the channel just this once')
+                        help="""Override the channel just this once.  Use in
+                                combination with `--build 0` to switch
+                                channels.""")
     parser.add_argument('-d', '--device',
                         default=None, action='store',
                         help='Override the device name just this once')
@@ -85,10 +87,6 @@ def main():
                         default=False, action='store_true',
                         help="""Calculate and print the upgrade path, but do
                                 not download or apply it""")
-    parser.add_argument('-u', '--upgrade',
-                        default=None, metavar='NUMBER',
-                        help="""Upgrade from this build number instead of the
-                                system's current build number""")
     parser.add_argument('-v', '--verbose',
                         default=0, action='count',
                         help='Increase verbosity')
@@ -140,16 +138,28 @@ def main():
         config.device = args.device
 
     if args.info:
-        print(dedent("""\
-            current build number: {}
-            device name: {}
-            channel: {}
-            last update: {}""").format(
-                config.build_number,
-                config.device,
-                config.channel,
-                last_update_date(),
-                ))
+        alias = getattr(config.service, 'channel_target', None)
+        kws = dict(
+            build_number=config.build_number,
+            device=config.device,
+            channel=config.channel,
+            last_update=last_update_date(),
+            )
+        if alias is None:
+            template = """\
+                current build number: {build_number}
+                device name: {device}
+                channel: {channel}
+                last update: {last_update}"""
+        else:
+            template = """\
+                current build number: {build_number}
+                device name: {device}
+                channel: {channel}
+                alias: {alias}
+                last update: {last_update}"""
+            kws['alias'] = alias
+        print(dedent(template).format(**kws))
         # If there's additional version details, print this out now too.  We
         # sort the keys in reverse order because we want 'ubuntu' to generally
         # come first.
@@ -175,8 +185,25 @@ def main():
         # We probably won't get here..
         return 0
 
-    def callback(received, total):
-        log.debug('received: {} of {} bytes', received, total)
+    # When verbosity is at 1, logging every progress signal from
+    # ubuntu-download-manager would be way too noisy.  OTOH, not printing
+    # anything leads some folks to think the process is just hung, since it
+    # can take a long time to download all the data files.  As a compromise,
+    # we'll output some dots to stderr at verbosity 1, but we won't log these
+    # dots since they would just be noise.  This doesn't have to be perfect.
+    if args.verbose == 1:
+        dot_count = 0
+        def callback(received, total):
+            nonlocal dot_count
+            sys.stderr.write('.')
+            sys.stderr.flush()
+            dot_count += 1
+            if dot_count % 78 == 0 or received >= total:
+                sys.stderr.write('\n')
+                sys.stderr.flush()
+    else:
+        def callback(received, total):
+            log.debug('received: {} of {} bytes', received, total)
 
     DBusGMainLoop(set_as_default=True)
     state = State(candidate_filter=candidate_filter)
@@ -188,7 +215,16 @@ def main():
             print('Already up-to-date')
         else:
             winning_path = [str(image.version) for image in state.winner]
-            print('Upgrade path is {}'.format(COLON.join(winning_path)))
+            kws = dict(path=COLON.join(winning_path))
+            if state.channel_switch is None:
+                # We're not switching channels due to an alias change.
+                template = 'Upgrade path is {path}'
+            else:
+                # This upgrade changes the channel that our alias is mapped
+                # to, so include that information in the output.
+                template = 'Upgrade path is {path} ({from} -> {to})'
+                kws['from'], kws['to'] = state.channel_switch
+            print(template.format(**kws))
         return
     else:
         # Run the state machine to conclusion.  Suppress all exceptions, but
@@ -201,6 +237,7 @@ def main():
         except KeyboardInterrupt:
             return 0
         except:
+            log.exception('system-image-cli exception')
             return 1
         else:
             return 0
