@@ -22,12 +22,15 @@ __all__ = [
 
 
 import os
+import json
 import shutil
 import hashlib
 import logging
+import tarfile
 
 from collections import deque
 from contextlib import ExitStack
+from datetime import datetime, timezone
 from functools import partial
 from itertools import islice
 from systemimage.candidates import get_candidates, iter_path
@@ -35,7 +38,8 @@ from systemimage.channel import Channels
 from systemimage.config import config
 from systemimage.download import DBusDownloadManager
 from systemimage.gpg import Context, SignatureError
-from systemimage.helpers import makedirs, safe_remove
+from systemimage.helpers import (
+    atomic, makedirs, safe_remove, temporary_directory)
 from systemimage.index import Index
 from systemimage.keyring import KeyringError, get_keyring
 from urllib.parse import urljoin
@@ -56,6 +60,37 @@ def _copy_if_missing(src, dstdir):
         shutil.copy(src, dstdir)
 
 
+def _use_cached(txt, asc, keyrings, checksum=None, blacklist=None):
+    if not os.path.exists(txt) or not os.path.exists(asc):
+        return False
+    with Context(*keyrings, blacklist=blacklist) as ctx:
+        if not ctx.verify(asc, txt):
+            return False
+    if checksum is None:
+        return True
+    with open(txt, 'rb') as fp:
+        got = hashlib.sha256(fp.read()).hexdigest()
+        return got == checksum
+
+
+def _use_cached_keyring(txz, asc, signing_key):
+    if not _use_cached(txz, asc, (signing_key,)):
+        return False
+    # Do one additional check: unpack the .tar.xz file, grab the keyring.json
+    # and if it has an expiry key, make sure that the keyring has not expired.
+    with temporary_directory(dir=config.tempdir) as tmp:
+        with tarfile.open(txz, 'r:xz') as tf:
+            tf.extractall(tmp)
+        json_path = os.path.join(tmp, 'keyring.json')
+        with open(json_path, 'r', encoding='utf-8') as fp:
+            data = json.load(fp)
+    expiry = data.get('expiry')
+    timestamp = datetime.now(tz=timezone.utc).timestamp()
+    # We can use this keyring if it never expires, or if the expiration date
+    # is some time in the future.
+    return expiry is None or expiry > timestamp
+
+
 class State:
     def __init__(self, candidate_filter=None):
         # Variables which manage state transitions.
@@ -68,6 +103,8 @@ class State:
         self.index = None
         self.winner = None
         self.files = []
+        self.channel_switch = None
+        # Other public attributes.
         self.downloader = DBusDownloadManager()
         self._next.append(self._cleanup)
 
@@ -144,20 +181,11 @@ class State:
     def _cleanup(self):
         """Clean up the destination directories.
 
-        Removes all residual files from the cache and data partitions except
-        `log` and `last_log` in the cache partition.
+        Removes all residual files from the data partition.  We leave the
+        cache partition alone because some of those data files may still be
+        valid and we want to avoid re-downloading them if possible.
         """
-        cache_dir = config.updater.cache_partition
         data_dir = config.updater.data_partition
-        try:
-            files = os.listdir(cache_dir)
-        except FileNotFoundError:
-            pass
-        else:
-            for filename in files:
-                if filename in ('log', 'last_log'):
-                    continue
-                safe_remove(os.path.join(cache_dir, filename))
         # Remove only the blacklist files (and generic keyring files) since
         # these are the only ones that will be downloaded to this location.
         safe_remove(os.path.join(data_dir, 'blacklist.tar.xz'))
@@ -168,12 +196,15 @@ class State:
 
     def _get_blacklist_1(self):
         """First try to get the blacklist."""
-        # If there is no image master key, download one now.  Don't worry if
-        # we have an out of date key; that will be handled elsewhere.  The
-        # archive master key better be pre-installed (we cannot download it).
-        # Let any exceptions in grabbing the image master key percolate up.
-        if not os.path.exists(config.gpg.image_master):
-            log.info('No image master key found, downloading')
+        # If there is no image master key, or if the signature on the key is
+        # not valid, download one now.  Don't worry if we have an out of date
+        # key; that will be handled elsewhere.  The archive master key better
+        # be pre-installed (we cannot download it).  Let any exceptions in
+        # grabbing the image master key percolate up.
+        image_master = config.gpg.image_master
+        if not _use_cached_keyring(image_master, image_master + '.asc',
+                                   config.gpg.archive_master):
+            log.info('No valid image master key found, downloading')
             get_keyring(
                 'image-master', 'gpg/image-master.tar.xz', 'archive-master')
         # The only way to know whether there is a blacklist or not is to try
@@ -182,8 +213,8 @@ class State:
         try:
             # I think it makes no sense to check the blacklist when we're
             # downloading a blacklist file.
-            log.info('Looking for blacklist: {}',
-                     urljoin(config.service.https_base, url))
+            log.info('Looking for blacklist: {}'.format(
+                     urljoin(config.service.https_base, url)))
             get_keyring('blacklist', url, 'image-master')
         except SignatureError:
             log.info('No signed blacklist found')
@@ -200,7 +231,10 @@ class State:
             self.blacklist = os.path.join(
                 config.updater.data_partition, 'blacklist.tar.xz')
             log.info('Local blacklist file: {}', self.blacklist)
-        self._next.append(self._get_channel)
+        # This is the first time we're trying to get the channel.json file.
+        # If this fails because signature is invalid, we'll try to download a
+        # new image-signing key.  Then we'll call _get_channel() again.
+        self._next.append(partial(self._get_channel, 0))
 
     def _get_blacklist_2(self):
         """Second try to get the blacklist."""
@@ -226,18 +260,22 @@ class State:
             self.blacklist = os.path.join(
                 config.updater.data_partition, 'blacklist.tar.xz')
             log.info('Local blacklist file: {}', self.blacklist)
-        self._next.append(self._get_channel)
+        # See above.
+        self._next.append(partial(self._get_channel, 0))
 
-    def _get_channel(self):
+    def _get_channel(self, count):
         """Get and verify the channels.json file."""
         # If there is no image signing key, download one now.  Don't worry if
         # we have an out of date key; that will be handled elsewhere.  The
         # imaging signing must be signed by the image master key, which we
         # better already have an up-to-date copy of.
-        if not os.path.exists(config.gpg.image_signing):
-            log.info('No image signing key found, downloading')
+        image_signing = config.gpg.image_signing
+        if not _use_cached_keyring(image_signing, image_signing + '.asc',
+                                   config.gpg.image_master):
+            log.info('No valid image signing key found, downloading')
             get_keyring(
-                'image-signing', 'gpg/image-signing.tar.xz', 'image-master')
+                'image-signing', 'gpg/image-signing.tar.xz', 'image-master',
+                self.blacklist)
         channels_url = urljoin(config.service.https_base, 'channels.json')
         channels_path = os.path.join(config.tempdir, 'channels.json')
         asc_url = urljoin(config.service.https_base, 'channels.json.asc')
@@ -257,8 +295,12 @@ class State:
                 Context(config.gpg.image_signing, blacklist=self.blacklist))
             if not ctx.verify(asc_path, channels_path):
                 # The signature on the channels.json file did not match.
-                # Maybe there's a new image signing key on the server.  If a
-                # new key *is* found, retry the current step.
+                # Maybe there's a new image signing key on the server.  If
+                # we've already downloaded a new image signing key, then
+                # there's nothing more to do but raise an exception.
+                # Otherwise, if a new key *is* found, retry the current step.
+                if count > 0:
+                    raise SignatureError(channels_path)
                 self._next.appendleft(self._get_signing_key)
                 log.info('channels.json not properly signed')
                 return
@@ -308,10 +350,9 @@ class State:
                 'image-master', 'gpg/image-master.tar.xz',
                 'archive-master', self.blacklist)
         except (FileNotFoundError, SignatureError, KeyringError):
-            # No valid image master key could be found.  Don't chain this
-            # exception.
-            log.error('No valid imaging master key found')
-            raise SignatureError from None
+            # No valid image master key could be found.
+            log.error('No valid image master key found')
+            raise
         # Retry the previous step.
         log.info('Installing new image master key to: {}',
                  config.gpg.image_master)
@@ -330,9 +371,11 @@ class State:
         except (FileNotFoundError, SignatureError, KeyringError):
             # No valid image signing key could be found.  Don't chain this
             # exception.
-            raise SignatureError from None
-        # Retry the previous step.
-        self._next.appendleft(self._get_channel)
+            log.error('No valid image signing key found')
+            raise
+        # Retry the previous step, but signal to _get_channel() that if the
+        # signature fails this time, it's an error.
+        self._next.appendleft(partial(self._get_channel, 1))
 
     def _get_index(self, index):
         """Get and verify the index.json file."""
@@ -370,16 +413,22 @@ class State:
         # changed, squash the build number to 0 before calculating the
         # winner.  Otherwise, trust the configured build number.
         channel = self.channels[config.channel]
-        channel_alias = getattr(channel, 'alias', None)
+        # channel_target is the channel we're on based on the alias mapping in
+        # our channel.ini file.  channel_alias is the alias mapping in the
+        # channel.json file, i.e. the channel an update will put us on.
         channel_target = getattr(config.service, 'channel_target', None)
+        channel_alias = getattr(channel, 'alias', None)
         if (    channel_alias is None or
                 channel_target is None or
                 channel_alias == channel_target):
             build_number = config.build_number
-        else:
+        elif config.build_number_cli is not None:
             # An explicit --build on the command line still takes precedence.
-            build_number = (0 if config.build_number_cli is None
-                            else config.build_number_cli)
+            build_number = config.build_number_cli
+        else:
+            # This is a channel switch caused by a new alias.
+            build_number = 0
+            self.channel_switch = (channel_target, channel_alias)
         candidates = get_candidates(self.index, build_number)
         if self._filter is not None:
             candidates = self._filter(candidates)
@@ -396,44 +445,58 @@ class State:
 
     def _download_files(self):
         """Download and verify all the winning upgrade path's files."""
-        downloads = []
-        signatures = []
-        sizes = []
-        checksums = []
-        for image_number, filerec in iter_path(self.winner):
-            # Re-pack for arguments to get_files() and to collate the size,
-            # signature path, and checksum for the downloadable file.
-            dst = os.path.join(
-                config.updater.cache_partition, os.path.basename(filerec.path))
-            downloads.append((
-                urljoin(config.service.http_base, filerec.path),
-                dst,
-                ))
-            sizes.append(filerec.size)
-            self.files.append((dst, (image_number, filerec.order)))
-            # Add the signature file, and associate the two.
-            asc = os.path.join(
-                config.updater.cache_partition,
-                os.path.basename(filerec.signature))
-            downloads.append((
-                urljoin(config.service.http_base, filerec.signature),
-                asc))
-            # There is no size available for the .asc file.
-            sizes.append(0)
-            self.files.append((asc, (image_number, filerec.order)))
-            signatures.append((dst, asc))
-            checksums.append((dst, filerec.checksum))
         # If there is a device-signing key, the files can be signed by either
         # that or the image-signing key.
         keyrings = [config.gpg.image_signing]
         if os.path.exists(config.gpg.device_signing):
             keyrings.append(config.gpg.device_signing)
-        # We must make sure that none of the destination file paths exist,
-        # otherwise the downloader will throw exceptions.
+        # Now, go through all the file records in the winning upgrade path.
+        # If the data file has already been downloaded and it has a valid
+        # signature file, then we can save some bandwidth by not downloading
+        # it again.
+        downloads = []
+        signatures = []
+        checksums = []
+        # For the clean ups below, preserve recovery's log files.
+        cache_dir = config.updater.cache_partition
+        preserve = set((
+            os.path.join(cache_dir, 'log'),
+            os.path.join(cache_dir, 'last_log'),
+            ))
+        for image_number, filerec in iter_path(self.winner):
+            # Re-pack for arguments to get_files() and to collate the
+            # signature path and checksum for the downloadable file.
+            dst = os.path.join(cache_dir, os.path.basename(filerec.path))
+            asc = os.path.join(cache_dir, os.path.basename(filerec.signature))
+            checksum = filerec.checksum
+            self.files.append((dst, (image_number, filerec.order)))
+            self.files.append((asc, (image_number, filerec.order)))
+            # Check the existence and signature of the file.
+            if _use_cached(dst, asc, keyrings, checksum, self.blacklist):
+                preserve.add(dst)
+                preserve.add(asc)
+            else:
+                downloads.append((
+                    urljoin(config.service.http_base, filerec.path),
+                    dst,
+                    ))
+                downloads.append((
+                    urljoin(config.service.http_base, filerec.signature),
+                    asc))
+                signatures.append((dst, asc))
+                checksums.append((dst, checksum))
+        # For any files we're about to download, we must make sure that none
+        # of the destination file paths exist, otherwise the downloader will
+        # throw exceptions.
         for url, dst in downloads:
             safe_remove(dst)
-        # Now, download all the files, providing logging feedback on progress.
-        # This download can be paused.
+        # Also delete cache partition files that we no longer need.
+        for filename in os.listdir(cache_dir):
+            path = os.path.join(cache_dir, filename)
+            if path not in preserve:
+                safe_remove(os.path.join(cache_dir, filename))
+        # Now, download all missing or ill-signed files, providing logging
+        # feedback on progress.  This download can be paused.
         self.downloader.get_files(downloads, pausable=True)
         with ExitStack() as stack:
             # Set things up to remove the files if a SignatureError gets
@@ -445,7 +508,7 @@ class State:
             # Although we should never get there, if the downloading step
             # fails, clear out the self.files list so there's no possibilty
             # we'll try to move them later.
-            stack.callback(setattr, self.files, [])
+            stack.callback(setattr, self, 'files', [])
             # Verify the signatures on all the downloaded files.
             with Context(*keyrings, blacklist=self.blacklist) as ctx:
                 for dst, asc in signatures:
@@ -459,7 +522,7 @@ class State:
                         raise ChecksumError(dst, got, checksum)
             # Everything is fine so nothing needs to be cleared.
             stack.pop_all()
-        log.info('all files available in {}', config.updater.cache_partition)
+        log.info('all files available in {}', cache_dir)
         # Now, copy the files from the temporary directory into the location
         # for the upgrader.
         self._next.append(self._move_files)
@@ -518,7 +581,7 @@ class State:
         # Open command file and first write the load_keyring commands.
         command_file = os.path.join(
             config.updater.cache_partition, 'ubuntu_command')
-        with open(command_file, 'w', encoding='utf-8') as fp:
+        with atomic(command_file) as fp:
             print('load_keyring {0} {0}.asc'.format(
                 os.path.basename(config.gpg.image_master)),
                 file=fp)
