@@ -16,14 +16,15 @@
 """Test helpers."""
 
 __all__ = [
+    'ServerTestBase',
     'configuration',
     'copy',
     'data_path',
     'debug',
+    'find_dbus_process',
     'get_channels',
     'get_index',
     'make_http_server',
-    'patience',
     'reset_envar',
     'setup_index',
     'setup_keyring_txz',
@@ -36,14 +37,14 @@ __all__ = [
 import os
 import ssl
 import json
-import time
 import gnupg
+import psutil
 import shutil
 import inspect
 import tarfile
+import unittest
 
 from contextlib import ExitStack, contextmanager
-from datetime import datetime, timedelta
 from functools import partial, wraps
 from http.server import HTTPServer, SimpleHTTPRequestHandler
 from pkg_resources import resource_filename, resource_string as resource_bytes
@@ -52,11 +53,13 @@ from systemimage.channel import Channels
 from systemimage.config import Configuration, config
 from systemimage.helpers import atomic, makedirs, temporary_directory
 from systemimage.index import Index
+from systemimage.state import State
 from threading import Thread
 from unittest.mock import patch
 
 
 EMPTYSTRING = ''
+SPACE = ' '
 
 
 def get_index(filename):
@@ -286,7 +289,7 @@ def setup_keyring_txz(keyring_src, signing_keyring, json_data, dst):
         shutil.copy(tarxz_path + '.asc', dst + '.asc')
 
 
-def setup_keyrings(*keyrings, use_config=None):
+def setup_keyrings(*keyrings, use_config=None, **data):
     """Copy the named keyrings to the right place.
 
     Also, set up the .xz.tar and .xz.tar.asc files which must exist in order
@@ -298,6 +301,8 @@ def setup_keyrings(*keyrings, use_config=None):
         e.g. 'archive_master'.
     :param use_config: If given, use this as the config object, otherwise use
         the global config object.
+    :param data: Additional key/value data to insert into the keyring.json
+        dictionary.
     """
     if len(keyrings) == 0:
         keyrings = ('archive-master', 'image-master', 'image-signing',
@@ -318,6 +323,7 @@ def setup_keyrings(*keyrings, use_config=None):
                                 else use_config.tempdir))
         # Now set up the .tar.xz and .tar.xz.asc files in the destination.
         json_data = dict(type=keyring)
+        json_data.update(data)
         dst = getattr((config.gpg if use_config is None
                        else use_config.gpg),
                       keyring.replace('-', '_'))
@@ -336,8 +342,7 @@ def setup_index(index, todir, keyring):
                 os.path.splitext(filerec.path)[0].split('/'))
             with open(dst, 'w', encoding='utf-8') as fp:
                 fp.write(contents)
-            # Sign with the imaging signing key.  Some tests will
-            # re-sign all these files with the device key.
+            # Sign with the specified signing key.
             sign(dst, keyring)
 
 
@@ -358,6 +363,9 @@ def reset_envar(name):
 
 
 def touch_build(version, timestamp=None):
+    # LP: #1220238 - assert that no old-style version numbers are being used.
+    assert 0 <= version < (1 << 16), (
+        'old style version number: {}'.format(version))
     with open(config.system.build_file, 'w', encoding='utf-8') as fp:
         print(version, file=fp)
     if timestamp is not None:
@@ -374,17 +382,109 @@ def touch_build(version, timestamp=None):
 @contextmanager
 def debug():
     with open('/tmp/debug.log', 'a', encoding='utf-8') as fp:
-        yield partial(print, file=fp)
+        function = partial(print, file=fp)
+        function.fp = fp
+        yield function
+        fp.flush()
 
 
-@contextmanager
-def patience(exception):
-    until = datetime.now() + timedelta(seconds=60)
-    while datetime.now() < until:
-        time.sleep(0.1)
+def find_dbus_process(ini_path):
+    """Return the system-image-dbus process running the given ini file."""
+    # This method searches all processes for the one matching the
+    # system-image-dbus service.  This is harder than it should be because
+    # while dbus-launch gives us the PID of the dbus-launch process itself,
+    # that can't be used to find the appropriate child process, because
+    # D-Bus activated processes are orphaned to init as their parent.
+    #
+    # This then does a brute-force search over all the processes, looking one
+    # that has a particular command line indicating that it's the
+    # system-image-dbus service.  We don't run this latter by that name
+    # though, since that's a wrapper created by setup.py's entry points.
+    #
+    # To make doubly certain we're not going to get the wrong process (in case
+    # there are multiple system-image-dbus processes running), we'll also look
+    # for the specific ini_path for the instance we care about.  Yeah, this
+    # all kind of sucks, but should be effective in finding the one we need to
+    # track.
+    for process in psutil.process_iter():
+        cmdline = SPACE.join(process.cmdline)
+        if 'systemimage.service' in cmdline and ini_path in cmdline:
+            return process
+    return None
+
+
+class ServerTestBase(unittest.TestCase):
+    # Must override in base classes.
+    INDEX_FILE = None
+    CHANNEL_FILE = None
+    CHANNEL = None
+    DEVICE = None
+    SIGNING_KEY = 'device-signing.gpg'
+
+    # For more detailed output.
+    maxDiff = None
+
+    @classmethod
+    def setUpClass(self):
+        # Avoid circular imports.
+        from systemimage.testing.nose import SystemImagePlugin
+        SystemImagePlugin.controller.set_mode(cert_pem='cert.pem')
+
+    def setUp(self):
+        self._resources = ExitStack()
+        self._enter = self._resources.enter_context
+        self._state = State()
         try:
-            yield
-        except exception:
-            break
-    else:
-        raise RuntimeError('Process did not exit')
+            self._serverdir = self._enter(temporary_directory())
+            # Start up both an HTTPS and HTTP server.  The data files are
+            # vended over the latter, everything else, over the former.
+            self._resources.push(make_http_server(
+                self._serverdir, 8943, 'cert.pem', 'key.pem'))
+            self._resources.push(make_http_server(self._serverdir, 8980))
+            # Set up the server files.
+            assert self.CHANNEL_FILE is not None, (
+                'Subclasses must set CHANNEL_FILE')
+            copy(self.CHANNEL_FILE, self._serverdir, 'channels.json')
+            sign(os.path.join(self._serverdir, 'channels.json'),
+                 'image-signing.gpg')
+            assert self.CHANNEL is not None, 'Subclasses must set CHANNEL'
+            assert self.DEVICE is not None, 'Subclasses must set DEVICE'
+            index_path = os.path.join(
+                self._serverdir, self.CHANNEL, self.DEVICE, 'index.json')
+            head, tail = os.path.split(index_path)
+            assert self.INDEX_FILE is not None, (
+                'Subclasses must set INDEX_FILE')
+            copy(self.INDEX_FILE, head, tail)
+            sign(index_path, self.SIGNING_KEY)
+            setup_index(self.INDEX_FILE, self._serverdir, self.SIGNING_KEY)
+        except:
+            self._resources.close()
+            raise
+
+    def tearDown(self):
+        self._resources.close()
+
+    def _setup_server_keyrings(self, *, device_signing=True):
+        # Only the archive-master key is pre-loaded.  All the other keys
+        # are downloaded and there will be both a blacklist and device
+        # keyring.  The four signed keyring tar.xz files and their
+        # signatures end up in the proper location after the state machine
+        # runs to completion.
+        setup_keyrings('archive-master')
+        setup_keyring_txz(
+            'spare.gpg', 'image-master.gpg', dict(type='blacklist'),
+            os.path.join(self._serverdir, 'gpg', 'blacklist.tar.xz'))
+        setup_keyring_txz(
+            'image-master.gpg', 'archive-master.gpg',
+            dict(type='image-master'),
+            os.path.join(self._serverdir, 'gpg', 'image-master.tar.xz'))
+        setup_keyring_txz(
+            'image-signing.gpg', 'image-master.gpg',
+            dict(type='image-signing'),
+            os.path.join(self._serverdir, 'gpg', 'image-signing.tar.xz'))
+        if device_signing:
+            setup_keyring_txz(
+                'device-signing.gpg', 'image-signing.gpg',
+                dict(type='device-signing'),
+                os.path.join(self._serverdir, self.CHANNEL, self.DEVICE,
+                             'device-signing.tar.xz'))
