@@ -16,8 +16,11 @@
 """Test the main entry point."""
 
 __all__ = [
+    'TestCLIDuplicateDestinations',
+    'TestCLIFilters',
     'TestCLIMain',
     'TestCLIMainDryRun',
+    'TestCLIMainDryRunAliases',
     'TestDBusMain',
     ]
 
@@ -26,7 +29,6 @@ import os
 import dbus
 import stat
 import time
-import psutil
 import shutil
 import unittest
 
@@ -39,10 +41,9 @@ from systemimage.config import Configuration, config
 from systemimage.helpers import safe_remove
 from systemimage.main import main as cli_main
 from systemimage.testing.helpers import (
-    configuration, copy, data_path, temporary_directory, touch_build)
+    ServerTestBase, configuration, copy, data_path, find_dbus_process,
+    temporary_directory, touch_build)
 from systemimage.testing.nose import SystemImagePlugin
-# This should be moved and refactored.
-from systemimage.tests.test_state import _StateTestsBase
 from textwrap import dedent
 from unittest.mock import MagicMock, patch
 
@@ -62,30 +63,6 @@ def umask(new_mask):
             os.umask(old_mask)
 
 
-def _find_dbus_proc(ini_path):
-    # This method searches all processes for the one matching the
-    # system-image-dbus service.  This is harder than it should be because
-    # while dbus-launch gives us the PID of the dbus-launch process itself,
-    # that can't be used to find the appropriate child process, because
-    # D-Bus activated processes are orphaned to init as their parent.
-    #
-    # This then does a brute-force search over all the processes, looking one
-    # that has a particular command line indicating that it's the
-    # system-image-dbus service.  We don't run this latter by that name
-    # though, since that's a wrapper created by setup.py's entry points.
-    #
-    # To make doubly certain we're not going to get the wrong process (in case
-    # there are multiple system-image-dbus processes running), we'll also look
-    # for the specific ini_path for the instance we care about.  Yeah, this
-    # all kind of sucks, but should be effective in finding the one we need to
-    # track.
-    for process in psutil.process_iter():
-        cmdline = SPACE.join(process.cmdline)
-        if 'systemimage.service' in cmdline and ini_path in cmdline:
-            return process
-    return None
-
-
 class TestCLIMain(unittest.TestCase):
     def setUp(self):
         super().setUp()
@@ -219,10 +196,10 @@ class TestCLIMain(unittest.TestCase):
         # Set up the build number.
         config = Configuration()
         config.load(ini_file)
-        touch_build(20130701, TIMESTAMP)
+        touch_build(1701, TIMESTAMP)
         cli_main()
         self.assertEqual(self._stdout.getvalue(), dedent("""\
-            current build number: 20130701
+            current build number: 1701
             device name: nexus7
             channel: stable
             last update: 2013-08-01 12:11:10
@@ -241,14 +218,14 @@ class TestCLIMain(unittest.TestCase):
         # Set up the build number.
         config = Configuration()
         config.load(ini_file)
-        touch_build(20130701)
+        touch_build(1701)
         timestamp_1 = int(datetime(2011, 1, 8, 2, 3, 4).timestamp())
         os.utime(config.system.build_file, (timestamp_1, timestamp_1))
         timestamp_2 = int(datetime(2011, 8, 1, 5, 6, 7).timestamp())
         os.utime(channel_ini, (timestamp_2, timestamp_2))
         cli_main()
         self.assertEqual(self._stdout.getvalue(), dedent("""\
-            current build number: 20130833
+            current build number: 1833
             device name: nexus7
             channel: proposed
             last update: 2011-08-01 05:06:07
@@ -265,13 +242,13 @@ class TestCLIMain(unittest.TestCase):
         # Set up the build number.
         config = Configuration()
         config.load(ini_file)
-        touch_build(20130701)
+        touch_build(1701)
         timestamp_1 = int(datetime(2011, 1, 8, 2, 3, 4).timestamp())
         os.utime(config.system.build_file, (timestamp_1, timestamp_1))
         self.assertFalse(os.path.exists(channel_ini))
         cli_main()
         self.assertEqual(self._stdout.getvalue(), dedent("""\
-            current build number: 20130701
+            current build number: 1701
             device name: nexus7
             channel: stable
             last update: 2011-01-08 02:03:04
@@ -282,7 +259,7 @@ class TestCLIMain(unittest.TestCase):
         # -b/--build overrides the build number.
         config = Configuration()
         config.load(ini_file)
-        touch_build(20130701, TIMESTAMP)
+        touch_build(1701, TIMESTAMP)
         # Use --build to override the default build number.
         self._resources.enter_context(
             patch('systemimage.main.sys.argv',
@@ -302,7 +279,7 @@ class TestCLIMain(unittest.TestCase):
         # -d/--device overrides the device type.
         config = Configuration()
         config.load(ini_file)
-        touch_build(20130701, TIMESTAMP)
+        touch_build(1701, TIMESTAMP)
         self._resources.enter_context(
             patch('systemimage.main.sys.argv',
                   ['argv0', '-C', ini_file,
@@ -310,7 +287,7 @@ class TestCLIMain(unittest.TestCase):
                    '--info']))
         cli_main()
         self.assertEqual(self._stdout.getvalue(), dedent("""\
-            current build number: 20130701
+            current build number: 1701
             device name: phablet
             channel: stable
             last update: 2013-08-01 12:11:10
@@ -321,7 +298,7 @@ class TestCLIMain(unittest.TestCase):
         # -c/--channel overrides the channel.
         config = Configuration()
         config.load(ini_file)
-        touch_build(20130701, TIMESTAMP)
+        touch_build(1701, TIMESTAMP)
         self._resources.enter_context(
             patch('systemimage.main.sys.argv',
                   ['argv0', '-C', ini_file,
@@ -329,18 +306,38 @@ class TestCLIMain(unittest.TestCase):
                    '--info']))
         cli_main()
         self.assertEqual(self._stdout.getvalue(), dedent("""\
-            current build number: 20130701
+            current build number: 1701
             device name: nexus7
             channel: daily-proposed
             last update: 2013-08-01 12:11:10
             """))
 
+    @configuration
+    def test_channel_name_with_alias(self, ini_file):
+        # When the current channel has an alias, this is reflected in the
+        # output for --info
+        channel_ini = os.path.join(os.path.dirname(ini_file), 'channel.ini')
+        head, tail = os.path.split(channel_ini)
+        copy('channel_05.ini', head, tail)
+        touch_build(300, TIMESTAMP)
+        self._resources.enter_context(
+            patch('systemimage.main.sys.argv',
+                  ['argv0', '-C', ini_file, '--info']))
+        cli_main()
+        self.assertEqual(self._stdout.getvalue(), dedent("""\
+            current build number: 300
+            device name: nexus7
+            channel: daily
+            alias: saucy
+            last update: 2013-08-01 12:11:10
+            """))
+
     @configuration
     def test_all_overrides(self, ini_file):
         # Use -b -d and -c together.
         config = Configuration()
         config.load(ini_file)
-        touch_build(20130701, TIMESTAMP)
+        touch_build(1701, TIMESTAMP)
         # Use --build to override the default build number.
         self._resources.enter_context(
             patch('systemimage.main.sys.argv',
@@ -380,10 +377,10 @@ class TestCLIMain(unittest.TestCase):
         # Set up the build number.
         config = Configuration()
         config.load(ini_file)
-        touch_build(20130701, TIMESTAMP)
+        touch_build(1701, TIMESTAMP)
         cli_main()
         self.assertEqual(self._stdout.getvalue(), dedent("""\
-            current build number: 20130833
+            current build number: 1833
             device name: nexus7
             channel: proposed
             last update: 2013-08-01 12:11:10
@@ -403,7 +400,7 @@ class TestCLIMain(unittest.TestCase):
                   ['argv0', '-C', ini_file, '-i']))
         cli_main()
         self.assertEqual(self._stdout.getvalue(), dedent("""\
-            current build number: 20130833
+            current build number: 1833
             device name: nexus7
             channel: proposed
             last update: 2013-08-01 12:11:10
@@ -460,7 +457,7 @@ class TestCLIMain(unittest.TestCase):
                   ['argv0', '-C', ini_file, '-i']))
         cli_main()
         self.assertEqual(self._stdout.getvalue(), dedent("""\
-            current build number: 20130833
+            current build number: 1833
             device name: nexus7
             channel: proposed
             last update: 2013-08-01 12:11:10
@@ -481,14 +478,14 @@ class TestCLIMain(unittest.TestCase):
                   ['argv0', '-C', ini_file, '-i']))
         cli_main()
         self.assertEqual(self._stdout.getvalue(), dedent("""\
-            current build number: 20130833
+            current build number: 1833
             device name: nexus7
             channel: proposed
             last update: 2013-08-01 12:11:10
             """))
 
 
-class TestCLIMainDryRun(_StateTestsBase):
+class TestCLIMainDryRun(ServerTestBase):
     INDEX_FILE = 'index_14.json'
     CHANNEL_FILE = 'channels_06.json'
     CHANNEL = 'stable'
@@ -497,113 +494,156 @@ class TestCLIMainDryRun(_StateTestsBase):
     @configuration
     def test_dry_run(self, ini_file):
         # `system-image-cli --dry-run` prints the winning upgrade path.
-        self._setup_keyrings()
-        with ExitStack() as stack:
-            # We patch builtin print() rather than sys.stdout because the
-            # latter can mess with pdb output should we need to trace through
-            # the code.
-            capture = StringIO()
-            stack.enter_context(
-                patch('builtins.print', partial(print, file=capture)))
-            stack.enter_context(
-                patch('systemimage.main.sys.argv',
-                      ['argv0', '-C', ini_file, '--dry-run']))
-            cli_main()
-            self.assertEqual(capture.getvalue(),
-                             'Upgrade path is 20130200:20130201:20130304\n')
+        self._setup_server_keyrings()
+        # We patch builtin print() rather than sys.stdout because the
+        # latter can mess with pdb output should we need to trace through
+        # the code.
+        capture = StringIO()
+        self._enter(patch('builtins.print', partial(print, file=capture)))
+        self._enter(patch('systemimage.main.sys.argv',
+                          ['argv0', '-C', ini_file, '--dry-run']))
+        cli_main()
+        self.assertEqual(capture.getvalue(),
+                         'Upgrade path is 1200:1201:1304\n')
 
     @configuration
     def test_dry_run_no_update(self, ini_file):
         # `system-image-cli --dry-run` when there are no updates available.
-        self._setup_keyrings()
-        with ExitStack() as stack:
-            # We patch builtin print() rather than sys.stdout because the
-            # latter can mess with pdb output should we need to trace through
-            # the code.
-            capture = StringIO()
-            stack.enter_context(
-                patch('builtins.print', partial(print, file=capture)))
-            stack.enter_context(
-                patch('systemimage.main.sys.argv',
-                      ['argv0', '-C', ini_file, '--dry-run']))
-            # Set up the build number.
-            config = Configuration()
-            config.load(ini_file)
-            touch_build(20130701)
-            cli_main()
-            self.assertEqual(capture.getvalue(), 'Already up-to-date\n')
+        self._setup_server_keyrings()
+        # We patch builtin print() rather than sys.stdout because the
+        # latter can mess with pdb output should we need to trace through
+        # the code.
+        capture = StringIO()
+        self._enter(patch('builtins.print', partial(print, file=capture)))
+        self._enter(patch('systemimage.main.sys.argv',
+                          ['argv0', '-C', ini_file, '--dry-run']))
+        # Set up the build number.
+        config = Configuration()
+        config.load(ini_file)
+        touch_build(1701)
+        cli_main()
+        self.assertEqual(capture.getvalue(), 'Already up-to-date\n')
 
     @configuration
     def test_dry_run_bad_channel(self, ini_file):
-        # 'system-image-cli --dry-run --channel <bad-channel>` should say it's
+        # `system-image-cli --dry-run --channel <bad-channel>` should say it's
         # already up-to-date.
-        self._setup_keyrings()
-        with ExitStack() as stack:
-            # We patch builtin print() rather than sys.stdout because the
-            # latter can mess with pdb output should we need to trace through
-            # the code.
-            capture = StringIO()
-            stack.enter_context(
-                patch('builtins.print', partial(print, file=capture)))
-            # Use --build to override the default build number.
-            stack.enter_context(
-                patch('systemimage.main.sys.argv',
-                      ['argv0', '-C', ini_file,
-                       '--channel', 'daily-proposed',
-                       '--dry-run']))
-            cli_main()
-            self.assertEqual(capture.getvalue(), 'Already up-to-date\n')
+        self._setup_server_keyrings()
+        # We patch builtin print() rather than sys.stdout because the
+        # latter can mess with pdb output should we need to trace through
+        # the code.
+        capture = StringIO()
+        self._enter(patch('builtins.print', partial(print, file=capture)))
+        # Use --build to override the default build number.
+        self._enter(patch('systemimage.main.sys.argv',
+                          ['argv0', '-C', ini_file,
+                           '--channel', 'daily-proposed',
+                           '--dry-run']))
+        cli_main()
+        self.assertEqual(capture.getvalue(), 'Already up-to-date\n')
 
 
-class TestCLIFilters(_StateTestsBase):
+class TestCLIMainDryRunAliases(ServerTestBase):
+    INDEX_FILE = 'index_20.json'
+    CHANNEL_FILE = 'channels_10.json'
+    CHANNEL = 'daily'
+    DEVICE = 'manta'
+
+    @configuration
+    def test_dry_run_with_channel_alias_switch(self, ini_file):
+        # `system-image-cli --dry-run` where the channel alias the device was
+        # on got switched should include this information.
+        self._setup_server_keyrings()
+        channel_ini = os.path.join(os.path.dirname(ini_file), 'channel.ini')
+        head, tail = os.path.split(channel_ini)
+        copy('channel_05.ini', head, tail)
+        capture = StringIO()
+        self._enter(patch('systemimage.device.check_output',
+                          return_value='manta'))
+        self._enter(patch('builtins.print', partial(print, file=capture)))
+        self._enter(patch('systemimage.main.sys.argv',
+                          ['argv0', '-C', ini_file, '--dry-run']))
+        cli_main()
+        self.assertEqual(
+            capture.getvalue(),
+            'Upgrade path is 200:201:304 (saucy -> tubular)\n')
+
+
+class TestCLIFilters(ServerTestBase):
     INDEX_FILE = 'index_15.json'
     CHANNEL_FILE = 'channels_06.json'
     CHANNEL = 'stable'
     DEVICE = 'nexus7'
 
+    maxDiff = None
+
     @configuration
     def test_filter_full(self, ini_file):
         # With --filter=full, only full updates will be considered.
-        self._setup_keyrings()
-        with ExitStack() as stack:
-            # We patch builtin print() rather than sys.stdout because the
-            # latter can mess with pdb output should we need to trace through
-            # the code.
-            capture = StringIO()
-            stack.enter_context(
-                patch('builtins.print', partial(print, file=capture)))
-            stack.enter_context(
-                patch('systemimage.main.sys.argv',
-                      ['argv0', '-C', ini_file, '--dry-run',
-                       '--filter', 'full']))
-            # Set up the build number.
-            config = Configuration()
-            config.load(ini_file)
-            touch_build(20120100)
-            cli_main()
-            self.assertEqual(capture.getvalue(), 'Already up-to-date\n')
+        self._setup_server_keyrings()
+        # We patch builtin print() rather than sys.stdout because the
+        # latter can mess with pdb output should we need to trace through
+        # the code.
+        capture = StringIO()
+        self._enter(patch('builtins.print', partial(print, file=capture)))
+        self._enter(patch('systemimage.main.sys.argv',
+                          ['argv0', '-C', ini_file, '--dry-run',
+                           '--filter', 'full']))
+        # Set up the build number.
+        config = Configuration()
+        config.load(ini_file)
+        touch_build(100)
+        cli_main()
+        self.assertMultiLineEqual(
+            capture.getvalue(), 'Already up-to-date\n')
 
     @configuration
     def test_filter_delta(self, ini_file):
         # With --filter=delta, only delta updates will be considered.
-        self._setup_keyrings()
-        with ExitStack() as stack:
-            # We patch builtin print() rather than sys.stdout because the
-            # latter can mess with pdb output should we need to trace through
-            # the code.
-            capture = StringIO()
-            stack.enter_context(
-                patch('builtins.print', partial(print, file=capture)))
-            stack.enter_context(
-                patch('systemimage.main.sys.argv',
-                      ['argv0', '-C', ini_file, '--dry-run',
-                       '--filter', 'delta']))
-            # Set up the build number.
-            config = Configuration()
-            config.load(ini_file)
-            touch_build(20120100)
-            cli_main()
-            self.assertEqual(capture.getvalue(), 'Upgrade path is 20130600\n')
+        self._setup_server_keyrings()
+        # We patch builtin print() rather than sys.stdout because the
+        # latter can mess with pdb output should we need to trace through
+        # the code.
+        capture = StringIO()
+        self._enter(patch('builtins.print', partial(print, file=capture)))
+        self._enter(patch('systemimage.main.sys.argv',
+                          ['argv0', '-C', ini_file, '--dry-run',
+                           '--filter', 'delta']))
+        # Set up the build number.
+        config = Configuration()
+        config.load(ini_file)
+        touch_build(100)
+        cli_main()
+        self.assertMultiLineEqual(
+            capture.getvalue(), 'Upgrade path is 1600\n')
+
+
+class TestCLIDuplicateDestinations(ServerTestBase):
+    INDEX_FILE = 'index_23.json'
+    CHANNEL_FILE = 'channels_06.json'
+    CHANNEL = 'stable'
+    DEVICE = 'nexus7'
+
+    @configuration
+    def test_duplicate_destinations(self, ini_file):
+        # index_23.json has the bug we saw in the wild in LP: #1250181.
+        # There, the server erroneously included a data file twice in two
+        # different images.  This can't happen and indicates a server
+        # problem.  The client must refuse to upgrade in this case, by raising
+        # an exception.
+        self._setup_server_keyrings()
+        self._enter(patch('systemimage.main.sys.argv',
+                          ['argv0', '-C', ini_file]))
+        exit_code = cli_main()
+        self.assertEqual(exit_code, 1)
+        # 2013-11-12 BAW: IWBNI we could assert something about the log
+        # output, since that contains a display of the duplicate destination
+        # paths and the urls that map to them, but that's difficult for
+        # several reasons, including that we can't really mock the log
+        # instance (it's a local variable to main(), and the output will
+        # contain stack traces and random paths.  I bet we could hack
+        # something in with doctest.OutputChecker.check_output(), but I'm not
+        # sure it's worth it.
 
 
 class TestDBusMain(unittest.TestCase):
@@ -644,9 +684,9 @@ class TestDBusMain(unittest.TestCase):
         # The dbus service automatically exits after a set amount of time.
         #
         # Nothing has been spawned yet.
-        self.assertIsNone(_find_dbus_proc(self.ini_path))
+        self.assertIsNone(find_dbus_process(self.ini_path))
         self._activate()
-        process = _find_dbus_proc(self.ini_path)
+        process = find_dbus_process(self.ini_path)
         self.assertTrue(process.is_running())
         # Now wait for the process to self-terminate.  If this times out
         # before the process exits, a TimeoutExpired exception will be
@@ -656,9 +696,9 @@ class TestDBusMain(unittest.TestCase):
 
     def test_service_keepalive(self):
         # Proactively calling methods on the service keeps it alive.
-        self.assertIsNone(_find_dbus_proc(self.ini_path))
+        self.assertIsNone(find_dbus_process(self.ini_path))
         self._activate()
-        process = _find_dbus_proc(self.ini_path)
+        process = find_dbus_process(self.ini_path)
         self.assertTrue(process.is_running())
         # Normally the process would exit after 3 seconds, but we'll keep it
         # alive for a bit.
